"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.globalAppRegistry = exports.AppRegistry = exports.isReduxStore = void 0;
const eventemitter3_1 = __importDefault(require("eventemitter3"));
function isReduxStore(store) {
    return (store &&
        typeof store === 'object' &&
        Object.prototype.hasOwnProperty.call(store, 'dispatch'));
}
exports.isReduxStore = isReduxStore;
class AppRegistry {
    constructor() {
        this._emitter = new eventemitter3_1.default();
        this.plugins = {};
    }
    static get AppRegistry() {
        return AppRegistry;
    }
    deregisterPlugin(name) {
        delete this.plugins[name];
        return this;
    }
    getPlugin(name) {
        return this.plugins[name];
    }
    registerPlugin(name, plugin) {
        this.plugins[name] = plugin;
        return this;
    }
    deactivate() {
        for (const [name, plugin] of Object.entries(this.plugins)) {
            plugin.deactivate?.();
            this.deregisterPlugin(name);
        }
        for (const event of this.eventNames()) {
            this.removeAllListeners(event);
        }
    }
    addListener(eventName, listener) {
        return this.on(eventName, listener);
    }
    emit(eventName, ...args) {
        return this._emitter.emit(eventName, ...args);
    }
    eventNames() {
        return this._emitter.eventNames();
    }
    listenerCount(eventName) {
        return this._emitter.listeners(eventName).length;
    }
    listeners(eventName) {
        return this._emitter.listeners(eventName);
    }
    on(eventName, listener) {
        this._emitter.on(eventName, listener);
        return this;
    }
    once(eventName, listener) {
        this._emitter.once(eventName, listener);
        return this;
    }
    removeListener(eventName, listener) {
        this._emitter.removeListener(eventName, listener);
        return this;
    }
    removeAllListeners(eventName) {
        this._emitter.removeAllListeners(eventName);
        return this;
    }
}
exports.AppRegistry = AppRegistry;
exports.globalAppRegistry = Object.freeze(new AppRegistry());
//# sourceMappingURL=app-registry.js.map