"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.registerHadronPlugin = exports.createServiceProvider = exports.createServiceLocator = exports.createActivateHelpers = void 0;
const react_1 = __importStar(require("react"));
const react_redux_1 = require("react-redux");
const app_registry_1 = require("./app-registry");
const react_context_1 = require("./react-context");
class ActivateHelpersImpl {
    constructor() {
        this.cleanupFns = new Set();
        this.deactivateController = new AbortController();
        this.on = (emitter, evt, fn) => {
            const voidFn = (...args) => void fn(...args);
            emitter.on(evt, voidFn);
            this.addCleanup(() => {
                emitter.removeListener(evt, voidFn);
            });
        };
        this.addCleanup = (fn) => {
            this.cleanupFns.add(fn);
        };
        this.cleanup = () => {
            for (const fn of this.cleanupFns.values()) {
                fn();
            }
        };
        this.addCleanup(() => {
            this.deactivateController.abort();
        });
    }
    get signal() {
        return this.deactivateController.signal;
    }
}
function createActivateHelpers() {
    return new ActivateHelpersImpl();
}
exports.createActivateHelpers = createActivateHelpers;
function LegacyRefluxProvider({ store, actions, children, }) {
    const storeRef = (0, react_1.useRef)(store);
    const [state, setState] = (0, react_1.useState)(() => {
        return storeRef.current.state;
    });
    react_1.default.useEffect(() => {
        const unsubscribe = storeRef.current.listen?.((updatedState) => setState({ ...updatedState }), null);
        return () => unsubscribe?.();
    }, []);
    return react_1.default.cloneElement(children, { store, actions, ...actions, ...state });
}
const defaultMockOptions = {
    pluginName: '$$root',
    mockedEnvironment: false,
    mockServices: {},
    disableChildPluginRendering: false,
};
const MockOptionsContext = react_1.default.createContext(defaultMockOptions);
const useMockOption = (key) => {
    return (0, react_1.useContext)(MockOptionsContext)[key];
};
let serviceLocationInProgress = false;
const kLocator = Symbol('CompassServiceLocatorFunction');
function createServiceLocator(fn, name = fn.name) {
    return Object.assign(function (...args) {
        if (!serviceLocationInProgress) {
            throw new Error(`Using service locator function "${name}" outside of the service location lifecycle. ` +
                `Make sure that service locator function is passed as a second argument to the registerHadronPlugin method and is not used directly in a React render method.`);
        }
        return fn.call(this, ...args);
    }, { [kLocator]: true });
}
exports.createServiceLocator = createServiceLocator;
function createServiceProvider(fn) {
    const displayName = `ServiceProvider(${fn.displayName ?? fn.name})`;
    const Provider = function (props) {
        let content;
        try {
            serviceLocationInProgress = true;
            content = fn(props);
        }
        finally {
            serviceLocationInProgress = false;
        }
        return react_1.default.createElement(react_1.default.Fragment, null, content);
    };
    Provider.displayName = displayName;
    return Provider;
}
exports.createServiceProvider = createServiceProvider;
function isServiceLocator(val) {
    return Object.prototype.hasOwnProperty.call(val, kLocator);
}
function useHadronPluginActivate(config, services, props) {
    const registryName = `${config.name}.Plugin`;
    const isMockedEnvironment = useMockOption('mockedEnvironment');
    const mockServices = useMockOption('mockServices');
    const globalAppRegistry = (0, react_context_1.useGlobalAppRegistry)();
    const localAppRegistry = (0, react_context_1.useLocalAppRegistry)();
    let serviceImpls;
    try {
        serviceLocationInProgress = true;
        serviceImpls = Object.fromEntries(Object.keys({
            ...(isMockedEnvironment ? mockServices : {}),
            ...services,
        }).map((key) => {
            const usingMockService = isMockedEnvironment && !!mockServices?.[key];
            if (!usingMockService &&
                services?.[key] &&
                !isServiceLocator(services[key])) {
                throw new Error(`Function "${services[key].name}" that is being passed as a service locator can not be identified as one. ` +
                    `Make sure that you created a service locator function using createServiceLocator helper function.`);
            }
            const svc = usingMockService ? mockServices[key] : services?.[key]();
            try {
                return [key, svc];
            }
            catch (err) {
                if (err &&
                    typeof err === 'object' &&
                    'message' in err &&
                    typeof err.message === 'string')
                    err.message += ` [locating service '${key}' for '${registryName}']`;
                throw err;
            }
        }));
    }
    finally {
        serviceLocationInProgress = false;
    }
    const [{ store, actions, context }] = (0, react_1.useState)(() => localAppRegistry.getPlugin(registryName) ??
        (() => {
            const plugin = config.activate(props, {
                globalAppRegistry,
                localAppRegistry,
                ...serviceImpls,
            }, createActivateHelpers());
            localAppRegistry.registerPlugin(registryName, plugin);
            return plugin;
        })());
    return { store, actions, context };
}
function registerHadronPlugin(config, services) {
    const Component = config.component;
    const Plugin = (props) => {
        const isMockedEnvironment = useMockOption('mockedEnvironment');
        const mockedPluginName = useMockOption('pluginName');
        const disableRendering = useMockOption('disableChildPluginRendering');
        if (isMockedEnvironment &&
            mockedPluginName !== config.name &&
            disableRendering) {
            return null;
        }
        const { store, actions, context } = useHadronPluginActivate(config, services, props);
        if ((0, app_registry_1.isReduxStore)(store)) {
            return (react_1.default.createElement(react_redux_1.Provider, { context: context, store: store },
                react_1.default.createElement(Component, { ...props })));
        }
        return (react_1.default.createElement(LegacyRefluxProvider, { store: store, actions: actions },
            react_1.default.createElement(Component, { ...props })));
    };
    return Object.assign(Plugin, {
        displayName: config.name,
        useActivate: (props) => {
            useHadronPluginActivate(config, services, props);
        },
        withMockServices(mocks = {}, options) {
            const { globalAppRegistry = new app_registry_1.AppRegistry(), localAppRegistry = globalAppRegistry, ...mockServices } = mocks;
            const mockOptions = {
                ...defaultMockOptions,
                mockedEnvironment: true,
                pluginName: config.name,
                mockServices,
                ...options,
            };
            return function MockPluginWithContext(props) {
                return (react_1.default.createElement(MockOptionsContext.Provider, { value: mockOptions },
                    react_1.default.createElement(react_context_1.GlobalAppRegistryContext.Provider, { value: globalAppRegistry },
                        react_1.default.createElement(react_context_1.AppRegistryProvider, { localAppRegistry: localAppRegistry },
                            react_1.default.createElement(Plugin, { ...props })))));
            };
        },
    });
}
exports.registerHadronPlugin = registerHadronPlugin;
//# sourceMappingURL=register-plugin.js.map